import base64
import logging
import re
import six
from twisted.internet.defer import Deferred

import scrapy_rotated_proxy.signals as proxy_signals

from itertools import cycle

import time

from scrapy.utils.misc import load_object
from scrapy_rotated_proxy import util
from twisted.internet import defer
from twisted.internet import task
from twisted.internet import reactor
from six.moves.urllib.parse import urlunparse
from six.moves.urllib.request import proxy_bypass
from six.moves.urllib.parse import unquote
from scrapy import signals
from scrapy.exceptions import NotConfigured, CloseSpider
from scrapy.http import Request
from scrapy.settings import Settings
from scrapy.utils.httpobj import urlparse_cached
from scrapy.utils.python import to_bytes
from scrapy_rotated_proxy.extensions import default_settings

if six.PY2:
    from urllib2 import _parse_proxy
else:
    from urllib.request import _parse_proxy

logger = logging.getLogger(__name__)


class RotatedProxyMiddleware(object):
    @classmethod
    def from_crawler(cls, crawler):
        if not crawler.settings.getbool('ROTATED_PROXY_ENABLED'):
            raise NotConfigured

        auth_encoding = crawler.settings.get('HTTPPROXY_AUTH_ENCODING')
        o = cls(crawler, auth_encoding)
        crawler.signals.connect(o.spider_opened, signal=signals.spider_opened)
        crawler.signals.connect(o.spider_closed, signal=signals.spider_closed)
        crawler.signals.connect(o.proxy_block_received,
                                signal=proxy_signals.proxy_block)
        crawler.signals.connect(o.proxy_remove_received,
                                signal=proxy_signals.proxy_remove)
        return o

    def __init__(self, crawler, auth_encoding='latin-1'):
        self.crawler = crawler
        self.auth_encoding = auth_encoding
        self.proxies_storage = load_object(
            crawler.settings.get(
                'PROXY_STORAGE',
                getattr(default_settings, 'PROXY_STORAGE')
            )
        )(crawler.settings, self.auth_encoding)
        self.sleep_interval = crawler.settings.get('PROXY_SLEEP_INTERVAL',
                                                   getattr(default_settings, 'PROXY_SLEEP_INTERVAL'))
        self.spider_close_when_no_proxy = crawler.settings.get(
            'PROXY_SPIDER_CLOSE_WHEN_NO_PROXY',
            getattr(default_settings, 'PROXY_SPIDER_CLOSE_WHEN_NO_PROXY')
        )
        self.enable_reload_proxy = crawler.settings.get(
            'PROXY_RELOAD_ENABLED',
            getattr(default_settings, 'PROXY_RELOAD_ENABLED')
        )
        self.check_task = None
        self.spider = None
        self.proxies = None
        self.valid_proxies = {}
        self.black_queue = {}
        self.black_proxies = {}
        self.invalid_proxies = {}
        self.proxy_gen = {}

    @defer.inlineCallbacks
    def spider_opened(self, spider, *args, **kwargs):
        self.spider = spider
        self.proxies_storage.open_spider(spider)
        self.proxies = yield self.proxies_storage.proxies()
        self.check_task = task.LoopingCall(self._check_black_proxy)
        self.check_task.start(60)

        logger.info('{middleware} opened {backend}'.format(
            middleware=self.__class__.__name__,
            backend='with backend: {}'.format(self.proxies_storage.__class__.__name__)
        ))
        for scheme, proxies in self.proxies.items():
            logger.info(
                'Loaded {count} {scheme} proxy from {backend}'.format(
                    count=len(proxies),
                    scheme=scheme,
                    backend=self.proxies_storage.__class__.__name__
                ))

    def spider_closed(self, spider):
        self.proxies_storage.close_spider(spider)
        if self.check_task and self.check_task.running:
            self.check_task.stop()

        logger.info('{middleware} closed'.format(middleware=self.__class__.__name__))

    @defer.inlineCallbacks
    def process_request(self, request, spider):
        # When Retry, dont_filter=True, reset proxy
        if 'proxy' in request.meta:
            if request.meta['proxy'] is None:
                return
            creds, proxy_url = self._get_proxy(request.meta['proxy'], '')
            request.meta['proxy'] = proxy_url
            if creds and not request.headers.get('Proxy-Authorization'):
                request.headers['Proxy-Authorization'] = b'Basic ' + creds
            return
        elif not self.proxies:
            return

        parsed = urlparse_cached(request)
        scheme = parsed.scheme

        if scheme in ('http', 'https') and proxy_bypass(parsed.hostname):
            return

        if scheme in self.proxies:
            yield self._set_proxy(request, scheme)

    def _get_proxy(self, url, orig_type=''):
        proxy_type, user, password, hostport = _parse_proxy(url)
        proxy_url = urlunparse(
            (proxy_type or orig_type, hostport, '', '', '', ''))

        creds = util._basic_auth_header(user, password, self.auth_encoding) \
            if user else None

        return creds, proxy_url

    @defer.inlineCallbacks
    def _set_proxy(self, request, scheme):
        while True:
            creds, proxy = next(self._cycle_proxy(scheme))
            if not proxy:
                yield self.sleep(self.sleep_interval,
                                 'Proxy pool of {scheme} is empty, waiting '
                                 '{sleep_second} seconds'.format(
                                     scheme=scheme,
                                     sleep_second=self.sleep_interval
                                 ))
            else:
                break

        request.meta['proxy'] = proxy
        if creds:
            request.headers['Proxy-Authorization'] = b'Basic ' + creds

    def reload_proxies(self, proxies):
        self.proxies = proxies
        for scheme, proxies in self.proxies.items():
            logger.info(
                'Reloaded {count} {scheme} proxy from {backend}'.format(
                    count=len(proxies),
                    scheme=scheme,
                    backend=self.proxies_storage.__class__.__name__
                ))

    def _cycle_proxy(self, scheme):
        if not self.proxy_gen.get(scheme):
            self.proxy_gen[scheme] = self._gen_proxy(scheme)
        return self.proxy_gen[scheme]

    def _gen_proxy(self, scheme):
        while True:
            self.valid_proxies[scheme] = self.proxies[scheme]\
                                         - self.black_proxies.get(scheme, set())\
                                         - self.invalid_proxies.get(scheme, set())
            if not self.valid_proxies[scheme]:
                if self.spider_close_when_no_proxy:
                    self.crawler.engine.close_spider(self.spider,
                                                     'Run out of All Proxy')
                    break
                else:
                    logger.info(
                        'Run out of all {scheme} proxies'.format(scheme=scheme))
                    yield None, None

            logger.info('Left {count} {scheme} proxy to run'.format(
                count=len(self.valid_proxies[scheme]),
                scheme=scheme
            ))
            for proxy_item in self.valid_proxies[scheme]:
                if proxy_item in self.black_proxies.get(scheme, set()) or \
                        proxy_item in self.invalid_proxies.get(scheme, set()):
                    continue
                yield proxy_item

            if self.enable_reload_proxy:
                if isinstance(self.proxies_storage.proxies, Deferred):
                    self.proxies_storage.proxies.addCallback(self.reload_proxies)
                else:
                    self.reload_proxies(self.proxies_storage.proxies())

    def _extract_proxy_from_request(self, request):
        if request.meta.get('proxy'):
            scheme = urlparse_cached(request)[0]
            if request.headers.get('Proxy-Authorization'):
                creds = request.headers.get('Proxy-Authorization')\
                    .split(b' ')[-1]
            else:
                creds = None
            proxy = request.meta.get('proxy')
            return scheme, creds, proxy
        else:
            return None, None, None

    def proxy_remove_received(self, spider, request, exception):
        scheme, creds, proxy = self._extract_proxy_from_request(request)
        if proxy:
            self._remove_invalid_proxy(scheme, creds, proxy)

            logger.info(
                'Remove proxy: {proxy}, Total remove {count} {scheme} proxy'.format(
                    proxy=request.meta.get('proxy'),
                    count=len(self.invalid_proxies[scheme]),
                    scheme=scheme
                ))

    def _remove_invalid_proxy(self, scheme, creds, proxy):
        if scheme not in self.invalid_proxies:
            self.invalid_proxies.setdefault(scheme, set())

        proxy_item = (creds, proxy)
        if proxy_item not in self.invalid_proxies[scheme]:
            self.invalid_proxies[scheme].add(proxy_item)

    def proxy_block_received(self, spider, response, exception):
        scheme, creds, proxy = self._extract_proxy_from_request(response.request)
        if proxy:
            self._add_black_proxy(scheme, creds, proxy)

            logger.info(
                'Block proxy: {proxy}, Total block {count} {scheme} proxy'.format(
                    proxy=response.meta.get('proxy'),
                    count=len(self.black_proxies[scheme]),
                    scheme=scheme
                ))

    def _add_black_proxy(self, scheme, creds, proxy):
        if scheme not in self.black_proxies:
            self.black_proxies.setdefault(scheme, set())
        if scheme not in self.black_queue:
            self.black_queue.setdefault(scheme, list())

        proxy_item = (creds, proxy)
        if proxy_item not in self.black_proxies[scheme]:
            self.black_proxies[scheme].add(proxy_item)
            self.black_queue[scheme].insert(0, (proxy_item, time.time()))


    def _check_black_proxy(self):
        for scheme in self.black_queue.keys():
            while len(self.black_queue[scheme]):
                if int(time.time()) - int(self.black_queue[scheme][-1][1]) < self.sleep_interval:
                    break
                proxy_item, timestamp = self.black_queue[scheme].pop()
                if proxy_item in self.black_proxies[scheme]:
                    self.black_proxies[scheme].remove(proxy_item)

    def sleep(self, secs, msg):
        logger.info(msg)
        d = defer.Deferred()
        reactor.callLater(secs, d.callback, None)
        return d




import base64
import six
from six.moves.urllib.parse import unquote
from scrapy.utils.python import to_bytes


def _basic_auth_header(username, password, auth_encoding=None):
    if six.PY2:
        username = username.encode(auth_encoding)
        password = password.encode(auth_encoding)
    user_pass = to_bytes(
        '{username}:{password}'.format(username=unquote(username),
                                       password=unquote(password)),
        encoding=auth_encoding
    )
    return base64.b64encode(user_pass).strip()



import logging
from itertools import cycle

from twisted.internet import defer
from six.moves.urllib.parse import urlunparse
from twisted.internet.defer import returnValue
from txmongo.connection import ConnectionPool
from txmongo import filter as qf

from scrapy_rotated_proxy import util
from scrapy_rotated_proxy.extensions import default_settings


logger = logging.getLogger(__name__)

class MongoDBProxyStorage():

    def __init__(self, settings, auth_encoding='latin-1'):
        self.settings = settings
        self.auth_encoding = auth_encoding
        db_key = 'PROXY_MONGODB_DB'
        coll_key = 'PROXY_MONGODB_COLL'
        index_key = 'PROXY_MONGODB_COLL_INDEX'
        self.db_uri = 'mongodb://{account}{path}{options}'.format(
            account=self._gen_mongo_account(),
            path=self._gen_mongo_path(),
            options=self._gen_mongo_option()
        )
        self.db_name = settings.get(db_key, getattr(default_settings, db_key))
        self.coll_name = settings.get(coll_key,
                                      getattr(default_settings, coll_key))
        if index_key:
            self.db_index = settings.get(index_key,
                                         getattr(default_settings, index_key))
        self._db_client = None
        self._db = None
        self._coll = None

    @defer.inlineCallbacks
    def open_spider(self, spider):
        self._db_client = yield ConnectionPool(self.db_uri)
        self._db = self._db_client[self.db_name]
        self._coll = self._db[self.coll_name]
        yield self._coll.find_one(timeout=True)
        for index in self.db_index:
            yield self._coll.create_index(qf.sort(index))
        logger.info('{storage} opened'.format(storage=self.__class__.__name__))

    @defer.inlineCallbacks
    def close_spider(self, spider):
        yield self._db_client.disconnect()
        logger.info('{storage} closed'.format(storage=self.__class__.__name__))

    @defer.inlineCallbacks
    def proxies(self):
        '''
        :return: Dict
            {'http': [(UserPassBase64, proxy1_url), (None, proxy2_url)], 'https': []}
        '''

        def _gen_proxy(scheme, host, port, user=None, password=None):
            proxy_url = urlunparse((scheme, '{host}:{port}'.format(host=host,
                                                                   port=port),
                                    '', '', '', ''))
            creds = util._basic_auth_header(user, password,
                                            self.auth_encoding) if user else None
            return creds, proxy_url

        res = {}
        for scheme in ['http', 'https']:
            proxy_list = yield self._coll.find({'scheme': scheme})
            res[scheme] = set(map(lambda proxy: _gen_proxy(proxy['scheme'],
                                                           proxy['ip'],
                                                           proxy['port'],
                                                           proxy['username'],
                                                           proxy['password']),
                                  proxy_list))
        returnValue(res)


    def _gen_mongo_account(self):
        username_key = 'PROXY_MONGODB_USERNAME'
        password_key = 'PROXY_MONGODB_PASSWORD'
        if all(map(lambda x: self.settings.get(x, getattr(default_settings, x)),
                   [username_key, password_key])):
            return '{username}:{password}@'.format(
                username=self.settings.get(username_key, getattr(default_settings, username_key)),
                password=self.settings.get(password_key, getattr(default_settings, password_key))
            )
        else:
            return ''

    def _gen_mongo_path(self):
        host_key = 'PROXY_MONGODB_HOST'
        port_key = 'PROXY_MONGODB_PORT'
        auth_db_key = 'PROXY_MONGODB_AUTH_DB'
        return '{host}:{port}/{auth_db}'.format(
            host=self.settings.get(host_key, getattr(default_settings, host_key)),
            port=self.settings.get(port_key, getattr(default_settings, port_key)),
            auth_db=self.settings.get(auth_db_key, getattr(default_settings, auth_db_key)) or ''
        )

    def _gen_mongo_option(self):
        option_prefix = 'PROXY_MONGODB_OPTIONS_'
        res = '&'.join(
            map(lambda x: '{option}={value}'.format(
                option=x[0].replace(option_prefix, '').lower(),
                value=x[1]
            ), filter(lambda x: x[0].startswith(option_prefix), self.settings.items()))
        )
        return '?' + res if res else ''



ROTATED_PROXY_ENABLED = True
PROXY_STORAGE = 'scrapy_rotated_proxy.extensions.file_storage.FileProxyStorage'
PROXY_FILE_PATH = ''
# PROXY_STORAGE = 'scrapy_rotated_proxy.extensions.mongodb_storage.MongoDBProxyStorage'
PROXY_MONGODB_HOST = '127.0.0.1'
PROXY_MONGODB_PORT = 27017
PROXY_MONGODB_USERNAME = None
PROXY_MONGODB_PASSWORD = None
PROXY_MONGODB_AUTH_DB = 'admin'
PROXY_MONGODB_DB = 'proxy_management'
PROXY_MONGODB_COLL = 'proxy'
PROXY_MONGODB_COLL_INDEX = []
PROXY_SLEEP_INTERVAL = 60*60*24
PROXY_SPIDER_CLOSE_WHEN_NO_PROXY = True
PROXY_RELOAD_ENABLED = False



import os
import sys
from functools import partial

from decorator import contextmanager
from twisted.trial.unittest import TestCase, SkipTest
from twisted.internet import defer

from scrapy_rotated_proxy.downloadmiddlewares.proxy import \
    RotatedProxyMiddleware
from scrapy.downloadermiddlewares.httpproxy import HttpProxyMiddleware
from scrapy.exceptions import NotConfigured
from scrapy.http import Response, Request
from scrapy.spiders import Spider
from scrapy.crawler import Crawler
from scrapy.settings import Settings

spider = Spider('foo')


class TestDefaultHeadersMiddleware(TestCase):
    failureException = AssertionError

    def setUp(self):
        self._oldenv = os.environ.copy()

    def tearDown(self):
        os.environ = self._oldenv

    @contextmanager
    def _middleware(self, spider, settings, **kwargs):
        _crawler = Crawler(spider, settings)
        if 'auth_encoding' in kwargs:
            _mw = RotatedProxyMiddleware(_crawler,
                                         auth_encoding=kwargs['auth_encoding'])
        else:
            _mw = RotatedProxyMiddleware(_crawler)
        _mw.spider_opened(spider)
        try:
            yield _mw
        finally:
            _mw.spider_closed(spider)

    def test_not_enabled(self):
        settings = Settings({'ROTATEDPROXY_ENABLED': False})
        crawler = Crawler(spider, settings)
        self.assertRaises(NotConfigured,
                          partial(RotatedProxyMiddleware.from_crawler, crawler))

    @defer.inlineCallbacks
    def test_no_setting_proxies(self):
        with self._middleware(spider, None) as mw:
            for url in ('http://e.com', 'https://e.com', 'file:///tmp/a'):
                req = Request(url)
                res = yield mw.process_request(req, spider)
                assert res is None
                self.assertEqual(req.url, url)
                self.assertEqual(req.meta, {})

    @defer.inlineCallbacks
    def test_setting_proxies(self):

        http_proxy = 'https://proxy.for.http:3128'
        https_proxy = 'http://proxy.for.https:8080'
        settings = Settings(
            {'HTTP_PROXIES': [http_proxy], 'HTTPS_PROXIES': [https_proxy]})

        with self._middleware(spider, settings) as mw:
            for url, proxy in [('http://e.com', http_proxy),
                               ('https://e.com', https_proxy),
                               ('file://tmp/a', None)]:
                req = Request(url)
                res = yield mw.process_request(req, spider)
                assert res is None
                self.assertEqual(req.url, url)
                self.assertEqual(req.meta.get('proxy'), proxy)

    @defer.inlineCallbacks
    def test_proxy_precedence_meta(self):
        settings = Settings({'HTTP_PROXIES': ['https://proxy.com']})
        with self._middleware(spider, settings) as mw:
            req = Request('http://scrapytest.org',
                          meta={'proxy': 'https://new.proxy:3128'})
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://new.proxy:3128'})

    @defer.inlineCallbacks
    def test_proxy_auth(self):
        settings = Settings({'HTTP_PROXIES': ['https://user:pass@proxy:3128']})
        with self._middleware(spider, settings) as mw:
            req = Request('http://scrapytest.org')
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic dXNlcjpwYXNz')
            # proxy from request.meta
            req = Request('http://scrapytest.org', meta={
                'proxy': 'https://username:password@proxy:3128'})
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic dXNlcm5hbWU6cGFzc3dvcmQ=')

    @defer.inlineCallbacks
    def test_proxy_auth_empty_passwd(self):
        settings = Settings({'HTTP_PROXIES': ['https://user:@proxy:3128']})
        with self._middleware(spider, settings) as mw:
            req = Request('http://scrapytest.org')
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic dXNlcjo=')
            # proxy from request.meta
            req = Request('http://scrapytest.org',
                          meta={'proxy': 'https://username:@proxy:3128'})
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic dXNlcm5hbWU6')

    @defer.inlineCallbacks
    def test_proxy_auth_encoding(self):
        # utf-8 encoding
        settings = Settings(
            {'HTTP_PROXIES': [u'https://m\u00E1n:pass@proxy:3128']})
        with self._middleware(spider, settings, auth_encoding='utf-8') as mw:
            req = Request('http://scrapytest.org')
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic bcOhbjpwYXNz')

            # proxy from request.meta
            req = Request('http://scrapytest.org',
                          meta={'proxy': u'https://\u00FCser:pass@proxy:3128'})
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic w7xzZXI6cGFzcw==')

        with self._middleware(spider, settings, auth_encoding='latin-1') as mw:
            # default latin-1 encoding
            req = Request('http://scrapytest.org')
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic beFuOnBhc3M=')

            # proxy from request.meta, latin-1 encoding
            req = Request('http://scrapytest.org',
                          meta={'proxy': u'https://\u00FCser:pass@proxy:3128'})
            res = yield mw.process_request(req, spider)
            assert res is None
            self.assertEqual(req.meta, {'proxy': 'https://proxy:3128'})
            self.assertEqual(req.headers.get('Proxy-Authorization'),
                             b'Basic /HNlcjpwYXNz')

    @defer.inlineCallbacks
    def test_proxy_already_seted(self):
        settings = Settings({'HTTP_PROXIES': ['https://proxy.for.http:3128']})
        with self._middleware(spider, settings) as mw:
            req = Request('http://noproxy.com', meta={'proxy': None})
            res = yield mw.process_request(req, spider)
            assert res is None
            assert 'proxy' in req.meta and req.meta['proxy'] is None

    @defer.inlineCallbacks
    def test_proxy_rotation(self):
        settings = Settings({'HTTP_PROXIES': ['https://proxy1.for.http:3128',
                                              'https://proxy2.for.http:3128']})
        with self._middleware(spider, settings) as mw:
            req1 = Request('http://scrapytest.org')
            req2 = Request('http://scrapytest.org')
            req3 = Request('http://scrapytest.org')
            res1 = yield mw.process_request(req1, spider)
            res2 = yield mw.process_request(req2, spider)
            res3 = yield mw.process_request(req3, spider)
            assert res1 is None
            assert res2 is None
            assert res3 is None
            res_list = [{'proxy': 'https://proxy1.for.http:3128'},
                        {'proxy': 'https://proxy2.for.http:3128'}]
            self.assertIn(req1.meta, res_list)
            self.assertIn(req2.meta, res_list)
            self.assertIn(req3.meta, res_list)
            self.assertNotEqual(req1.meta, req2.meta)
            self.assertEqual(req1.meta, req3.meta)

    @defer.inlineCallbacks
    def test_multi_scheme(self):
        settings = Settings({'HTTP_PROXIES': ['https://proxy1.for.http:3128'],
                             'HTTPS_PROXIES': ['https://proxy2.for.http:3128']})
        with self._middleware(spider, settings) as mw:
            req_http = Request('http://scrapytest.org')
            req_https = Request('https://scrapytest.org')
            res_http = yield mw.process_request(req_http, spider)
            res_https = yield mw.process_request(req_https, spider)
            assert res_http is None
            assert res_https is None
            self.assertEqual(req_http.meta,
                             {'proxy': 'https://proxy1.for.http:3128'})
            self.assertEqual(req_https.meta,
                             {'proxy': 'https://proxy2.for.http:3128'})



import re
import json
import six
from itertools import cycle

from scrapy.utils.misc import load_object
from scrapy_rotated_proxy.extensions import default_settings
from scrapy_rotated_proxy import util

import logging
if six.PY2:
    from urlparse import urlunparse
    from urllib2 import _parse_proxy
else:
    from urllib.parse import urlunparse
    from urllib.request import _parse_proxy

logger = logging.getLogger(__name__)


class FileProxyStorage():
    def __init__(self, settings, auth_encoding='latin-1'):
        self.file_path = None
        if settings.get('PROXY_FILE_PATH') or getattr(default_settings,
                                                      'PROXY_FILE_PATH'):
            self.file_path = settings.get('PROXY_FILE_PATH',
                                     getattr(default_settings,
                                             'PROXY_FILE_PATH'))
        else:
            self.settings = settings
        self.auth_encoding = auth_encoding

    def open_spider(self, spider):
        logger.info('{storage} opened'.format(storage=self.__class__.__name__))

    def close_spider(self, spider):
        logger.info('{storage} closed'.format(storage=self.__class__.__name__))

    def _get_proxy(self, url, orig_type=''):
        proxy_type, user, password, hostport = _parse_proxy(url)
        proxy_url = urlunparse(
            (proxy_type or orig_type, hostport, '', '', '', ''))

        creds = util._basic_auth_header(user, password,
                                        self.auth_encoding) if user else None

        return creds, proxy_url

    def proxies(self):
        pattern = re.compile(r'(?P<scheme>[A-Z]+)_PROXIES')

        def _filter(tuple_):
            m = pattern.match(tuple_[0])
            if m:
                scheme = m.group('scheme').lower()
                return scheme, {self._get_proxy(item, scheme) for item in
                                tuple_[1]}

        if self.file_path:
            self.settings = json.load(open(self.file_path))

        proxies = []
        for item in self.settings.items():
            pair = _filter(item)
            if pair:
                proxies.append(pair)
        return dict(proxies)



from os.path import dirname, join
from pkg_resources import parse_version
from setuptools import setup, find_packages, __version__ as setuptools_version


with open(join(dirname(__file__), 'scrapy_rotated_proxy/VERSION'), 'rb') as f:
    version = f.read().decode('ascii').strip()


extras_require = {}

setup(
    name='scrapy-rotated-proxy',
    version=version,
    url='https://github.com/xiaowangwindow/scrapy-rotated-proxy',
    description='A middleware to change proxy rotated for Scrapy',
    long_description=open('README.rst').read(),
    author='Alex Wang',
    maintainer='Alex Wang',
    maintainer_email='xiaowangwindow@163.com',
    license='BSD',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Framework :: Scrapy',
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'scrapy>=1.4.0'
    ],
    extras_require=extras_require,
)
